<?php
/**
 * Install Controller
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Install_Controller extends Me_Controller
{
    /** システム設定フォーム名 */
    const SYSTEM_SETTING_FORM = 'install_system_setting_form';
    /** ユーザー設定フォーム名 */
    const USER_FORM = 'install_user_form';
    /**
     * Controller名
     */
    protected $controller = 'Install';
    /**
     * Logicオブジェクト自動生成
     * @var boolean
     */
    protected $logic_load = false;

    /**
     * actionの前処理
     */
    public function prepend()
    {
        // セッションを開始
        session_start();

        // CSRF対策
        if ($this->check_token) {
            Me_Util::check_token();
        }
    }

    /**
     * インストール開始
     */
    public function index()
    {

    }

    /**
     * ライセンス表示
     */
    public function license()
    {
        $this->view = '/Install/license.html';
    }

    /**
     * 環境確認
     */
    public function env_check()
    {
        // 書き込み権限チェック
        $check_files = array(
            DATA_ROOT . '/config/user_config.ini',
            DATA_ROOT . '/db/cms.db',
            DATA_ROOT . '/template',
            DATA_ROOT . '/module',
        );
        $check_files = array_flip($check_files);
        foreach (array_keys($check_files) as $key) {
            $check_files[$key] = is_writable($key);
        }

        // モジュール確認
        $module_loaded = array();
        $module_loaded['mbstring'] = extension_loaded('mbstring');
        $module_loaded['pdo'] = extension_loaded('pdo');
        $module_loaded['pdo_sqlite'] = extension_loaded('pdo_sqlite');
        $module_loaded['gd'] = extension_loaded('gd');

        $this->set_data(compact('check_files', 'module_loaded'));
    }

    /**
     * システム設定
     */
    public function system_setting()
    {
        $system_setting = array();
        if (isset($_SESSION[self::SYSTEM_SETTING_FORM])) {
            $system_setting = $_SESSION[self::SYSTEM_SETTING_FORM];
        } else {
            // document_root
            $system_setting['document_root'] = $_SERVER['DOCUMENT_ROOT'];
            // site_path
            $site_path = dirname(dirname(dirname($_SERVER['SCRIPT_NAME'])));
            $site_path = str_replace('\\', '/', $site_path);
            $system_setting['site_path'] = $site_path;
            $_SESSION[self::SYSTEM_SETTING_FORM] = $system_setting;
        }

        $this->set_data(compact('system_setting'));
    }

    /**
     * システム設定登録処理
     */
    public function post_system_setting()
    {
        if (empty($_POST) || !isset($_SESSION[self::SYSTEM_SETTING_FORM])) {
            $this->redirect_path = '../Install/system_setting';
            return;
        }

        $errors = array();
        // $system_settingはフォームオブジェクトではなく配列のため参照渡しを指定する
        $system_setting = &$_SESSION[self::SYSTEM_SETTING_FORM];
        // document_root
        if (isset($_POST['document_root'])) {
            $system_setting['document_root'] = rtrim($_POST['document_root'], '/');
        }
        // site_path
        if (isset($_POST['site_path'])) {
            if (strlen($_POST['site_path']) > 1) {
                $system_setting['site_path'] = rtrim($_POST['site_path'], '/');
            } else {
                $system_setting['site_path'] = $_POST['site_path'];
            }
        }

        if (!empty($errors)) {
            // エラー発生
            $errors['message'] = 'エラーが発生しました！';
            $this->view = '/Install/system_setting.php';
            return;
        }
        // cms_path
        $system_setting['cms_path'] = dirname($_SERVER['SCRIPT_NAME']);
        $system_setting['upload_path'] = dirname(dirname($_SERVER['SCRIPT_NAME'])) . '/upload';

        // me_session_id を生成する
        $me_session_id = sha1(mt_rand());
        // 設定ファイルを書きだす
        // バッファリング制御
        ob_start();
        include('def/config/user_config.php');
        $user_config_ini = ob_get_contents();
        //バッファを削除
        ob_end_clean();
        Me_Util::write_file(DATA_ROOT . '/config/user_config.ini', $user_config_ini);

        $this->redirect_path = '../Install/user_setting';
        return;
    }

    /**
     * ユーザー設定
     */
    public function user_setting()
    {
        if (isset($_SESSION[self::USER_FORM])) {
            $form = $_SESSION[self::USER_FORM];
        } else {
            $form = new User_Form();
        }
        $this->set_data(compact('form'));
    }

    /**
     * ユーザー登録処理
     */
    public function post_user_setting()
    {
        $form = new User_Form();
        $mapping = array(
            'user_name' => 'user_name',
            'password' => 'password',
            'confirm_password' => 'confirm_password',
            'display_name' => 'display_name',
            'note' => 'note',
        );
        Me_Util::post_data_to_object($form, $mapping);
        $form->user_role = User_Role::ADMIN;

        $user_logic = new User_Logic;
        if ($user_logic->register_user($form, true)) {
            // 登録成功
            // ファイル登録
            // 未登録のページリストを取得
            $page_logic = new Page_Logic();
            list($page_list, $delete_list) = $page_logic->get_sync_list();
            // DBに登録
            $page_logic->insert_page_list($page_list);

            // Meinfo登録
            $meinfo_dao = new Meinfo_Dao();
            $meinfo = $meinfo_dao->select(1);
            $meinfo->schema_version = ME_SCHEMA_VERSION;
            $meinfo->install_date = time();
            $meinfo_dao->update($meinfo);

            // セッション変数を全て解除する
            $_SESSION = array();
            // セッションを切断するにはセッションクッキーも削除する。
            if (isset($_COOKIE[session_name()])) {
                setcookie(session_name(), '', time() - 42000, '/');
            }
            // セッションを破壊する
            session_destroy();

            $this->view = '/Install/congrats.php';
        } else {
            // 登録失敗
            $_SESSION[self::USER_FORM] = $form;
            $this->view = '/Install/user_setting.php';
            $this->set_data(compact('form'));
        }
    }
}